commit 170be5ae3ec5391b008971312f57cc7e0b8592ba
Author: Rémi Denis-Courmont <remi@remlab.net>
Date:   Wed Aug 19 23:42:14 2009 +0300

    Trivial benchmark for vlc_memset and vlc_memcpy

diff --git a/src/libvlc.c b/src/libvlc.c
index 9dc80c6..1a4768c 100644
--- a/src/libvlc.c
+++ b/src/libvlc.c
@@ -97,6 +97,8 @@
 # include <libkern/OSAtomic.h>
 #endif
 
+#import <CoreFoundation/CoreFoundation.h>
+
 #include <assert.h>
 
 /*****************************************************************************
@@ -287,6 +289,7 @@ libvlc_int_t * libvlc_InternalCreate( void )
     return p_libvlc;
 }
 
+#include <sys/mman.h>
 /**
  * Initialize a libvlc instance
  * This function initializes a previously allocated libvlc instance:
@@ -784,6 +787,86 @@ int libvlc_InternalInit( libvlc_int_t *p_libvlc, int i_argc,
      */
     priv->p_memcpy_module = module_need( p_libvlc, "memcpy", "$memcpy", false );
 
+    void *d;
+    const void *s;
+    const size_t size = 256*1024*1024;
+    s = mmap(NULL, size, PROT_READ, MAP_PRIVATE|MAP_ANON, -1, 0);
+    d = mmap(NULL, size, PROT_WRITE, MAP_PRIVATE|MAP_ANON, -1, 0);
+    assert (d != MAP_FAILED);
+
+    // First memset to burn the track
+    memset(d, 0, size);
+    barrier();
+
+    CFTimeInterval ns = 0;
+    CFTimeInterval os = 0;
+    CFTimeInterval esp_ns = 0;
+    CFTimeInterval esp_os = 0;
+    for (unsigned i = 0; i < 10; i++)
+    {
+        CFAbsoluteTime t1 = CFAbsoluteTimeGetCurrent();
+        memset(d, 0, size);
+        barrier();
+        CFAbsoluteTime t2 = CFAbsoluteTimeGetCurrent();
+        memset(d, 0, size);
+        barrier();
+        CFAbsoluteTime t3 = CFAbsoluteTimeGetCurrent();
+
+        CFTimeInterval diff;
+
+        diff = (t3 - t2) * 1000;
+        printf("memset run %d NS: %fms\n", i, diff);
+        ns += diff;
+        esp_ns += diff * diff;
+
+        diff = (t2 - t1) * 1000;
+        printf("memset run %d OS: %fms\n", i, diff);
+        os += diff;
+        esp_os += diff * diff;
+    }
+    os /= 10;
+    ns /= 10;
+    esp_os /= 10;
+    esp_ns /= 10;
+    printf("memset average NS: %fms Variance: %fms\n", ns, sqrt(esp_ns - ns * ns));
+    printf("memset average OS: %fms Variance: %fms\n", os, sqrt(esp_os - os * os));
+    printf("Per run diff: %fms\n", os - ns);
+
+    // First memset to burn the track
+    memcpy(d, s, size);
+    barrier();
+    
+    ns = os = esp_os = esp_ns = 0;
+    for (unsigned i = 0; i < 10; i++)
+    {
+        CFAbsoluteTime t1 = CFAbsoluteTimeGetCurrent();
+        memcpy(d, s, size);
+        barrier();
+        CFAbsoluteTime t2 = CFAbsoluteTimeGetCurrent();
+        memcpy(d, s, size);
+        barrier();
+        CFAbsoluteTime t3 = CFAbsoluteTimeGetCurrent();
+
+        CFTimeInterval diff;
+        
+        diff = (t3 - t2) * 1000;
+        printf("memcpy run %d NS: %fms\n", i, diff);
+        ns += diff;
+        esp_ns += diff * diff;
+
+        diff = (t2 - t1) * 1000;
+        printf("memcpy run %d OS: %fms\n", i, diff);
+        os += diff;
+        esp_os += diff * diff;
+    }
+    os /= 10;
+    ns /= 10;
+    esp_os /= 10;
+    esp_ns /= 10;
+    printf("memcpy average NS: %fms Variance: %fms\n", ns, sqrt(esp_ns - ns * ns));
+    printf("memcpy average OS: %fms Variance: %fms\n", os, sqrt(esp_os - os * os));
+    printf("Per run diff: %fms\n", os - ns);
+
     priv->b_stats = config_GetInt( p_libvlc, "stats" ) > 0;
     priv->i_timers = 0;
     priv->pp_timers = NULL;
